// Get current sensor readings when the page loads   
window.addEventListener('load', getReadings);   
 
// Create angle Chart 
var chartA = new Highcharts.Chart({ 
  chart:{  
    renderTo:'chart-angle'  
  }, 
  series: [ 
    { 
      name: 'iSpindel' 
    } 
  ], 
  title: {  
    text: undefined 
  }, 
  plotOptions: { 
    line: {  
      animation: false, 
      dataLabels: {  
        enabled: true  
      } 
    } 
  }, 
  xAxis: { 
    type: 'datetime', 
    dateTimeLabelFormats: { second: '%H:%M:%S' } 
  }, 
  yAxis: { 
    title: {  
      text: 'Angle'  
    } 
  }, 
  credits: { 
    enabled: false  
} 
}); 
// Create gravity Chart 
var chartG = new Highcharts.Chart({ 
  chart:{  
    renderTo:'chart-gravity'  
  }, 
  series: [ 
    { 
      name: 'iSpindel' 
    } 
  ], 
  title: {  
    text: undefined 
  }, 
  plotOptions: { 
    line: {  
      animation: false, 
      dataLabels: {  
        enabled: true  
      } 
    } 
  }, 
  xAxis: { 
    type: 'datetime', 
    dateTimeLabelFormats: { second: '%H:%M:%S' } 
  }, 
  yAxis: { 
    title: {  
      text: 'Gravity'  
    } 
  }, 
  credits: { 
    enabled: false  
} 
}); 
 
// Create Temperature Chart 
var chartT = new Highcharts.Chart({ 
  chart:{  
    renderTo:'chart-temperature'  
  }, 
  series: [ 
    { 
      name: 'iSpindel' 
    } 
  ], 
  title: {  
    text: undefined 
  }, 
  plotOptions: { 
    line: {  
      animation: false, 
      dataLabels: {  
        enabled: true  
      } 
    } 
  }, 
  xAxis: { 
    type: 'datetime', 
    dateTimeLabelFormats: { second: '%H:%M:%S' } 
  }, 
  yAxis: { 
    title: {  
      text: 'Temperature'  
    } 
  }, 
  credits: { 
    enabled: false  
} 
}); 
 
// Create Battery Chart 
var chartB = new Highcharts.Chart({ 
  chart:{  
    renderTo:'chart-battery'  
  }, 
  series: [ 
    { 
      name: 'iSpindel' 
    } 
  ], 
  title: {  
    text: undefined 
  }, 
  plotOptions: { 
    line: {  
      animation: false, 
      dataLabels: {  
        enabled: true  
      } 
    } 
  }, 
  xAxis: { 
    type: 'datetime', 
    dateTimeLabelFormats: { second: '%H:%M:%S' } 
  }, 
  yAxis: { 
    title: {  
      text: 'Volt'  
    } 
  }, 
  credits: { 
    enabled: false  
} 
}); 

//Plot gravity in the angle chart 
function plotAngle(value){ 
var x = (new Date()).getTime()
var y = Number(value); 
if(chartA.series[0].data.length > 40) { 
  chartA.series[0].addPoint([x, y], true, true, true); 
} else { 
  chartA.series[0].addPoint([x, y], true, false, true); 
} 
} 

//Plot gravity in the gravity chart 
function plotGravity(value){ 
var x = (new Date()).getTime()
var y = Number(value); 
if(chartG.series[0].data.length > 40) { 
  chartG.series[0].addPoint([x, y], true, true, true); 
} else { 
  chartG.series[0].addPoint([x, y], true, false, true); 
} 
} 

//Plot Temperature in the temperature chart 
function plotTemperature(value){ 
var x = (new Date()).getTime()
var y = Number(value); 
if(chartT.series[0].data.length > 40) { 
  chartT.series[0].addPoint([x, y], true, true, true); 
} else { 
  chartT.series[0].addPoint([x, y], true, false, true); 
} 
} 

//Plot Battery in the battery chart 
function plotBattery(value){ 
var x = (new Date()).getTime()
var y = Number(value); 
if(chartB.series[0].data.length > 40) { 
  chartB.series[0].addPoint([x, y], true, true, true); 
} else { 
  chartB.series[0].addPoint([x, y], true, false, true); 
} 
} 

// Function to get current readings on the web page when it loads  
function getReadings() { 
var xhr = new XMLHttpRequest(); 
xhr.onreadystatechange = function() { 
  if (this.readyState == 4 && this.status == 200) { 
    console.log("["+this.responseText.slice(0, -1)+"]");
    var myObj = JSON.parse("["+this.responseText.slice(0, -1)+"]"); 
    console.log(myObj); 
    var len = myObj.length; 
    if(len > 40) { 
      for(var i = len-40; i<len; i++){ 
        plotAngle(myObj[i].angle); 
        plotGravity(myObj[i].gravity); 
        plotTemperature(myObj[i].temperature); 
        plotBattery(myObj[i].battery);
      } 
    } 
    else { 
      for(var i = 0; i<len; i++){ 
        plotAngle(myObj[i].angle); 
        plotGravity(myObj[i].gravity); 
        plotTemperature(myObj[i].temperature); 
        plotBattery(myObj[i].battery);
      } 
    } 
  } 
};  
xhr.open("GET", "/view-data", true); 
xhr.send(); 
} 

if (!!window.EventSource) { 
var source = new EventSource('/events'); 
   
source.addEventListener('open', function(e) { 
  console.log("Events Connected"); 
}, false); 

source.addEventListener('error', function(e) { 
  if (e.target.readyState != EventSource.OPEN) { 
    console.log("Events Disconnected"); 
  } 
}, false); 
   
source.addEventListener('message', function(e) { 
  console.log("message", e.data); 
}, false); 
   
source.addEventListener('new_readings', function(e) { 
  console.log("new_readings", e.data); 
  var myObj = JSON.parse(e.data); 
  console.log(myObj); 
  plotAngle(myObj.angle); 
  plotGravity(myObj.gravity); 
  plotTemperature(myObj.temperature); 
  plotBattery(myObj.battery); 
}, false); 
}
