/**************************************************************

    "iMonitor"
    By Adrian S. <>

    "Data controller"

 **************************************************************/

#include "Data.h"

#define SD_CS 5

Data::Data() {}

// Gibt reduzierte Daten der iSindel weiter
String Data::setViewData(String data)
{
    // Erstellt einen JSON-Buffer
    StaticJsonDocument<200> viewData;

    // Erstellt den JSON-Dokument der Einstellungen
    DynamicJsonDocument dataDoc(1024);
    deserializeJson(dataDoc, data);

    // Fügt Einträge zum JSON-Buffer hinzu
    viewData["name"] = dataDoc["name"].as<String>();
    viewData["angle"] = String(dataDoc["angle"].as<float>(), 2);
    viewData["temperature"] = String(dataDoc["temperature"].as<float>(), 2);
    viewData["battery"] = String(dataDoc["battery"].as<float>(), 2);
    viewData["gravity"] = String(dataDoc["gravity"].as<float>(), 2);
    viewData["temp_units"] = dataDoc["temp_units"].as<String>();

    // Erstellt den JSON-String
    String dataString;
    serializeJson(viewData, dataString);

    // Serial Monitor Ausgabe
    // Serial.print("iSpindel last data :");
    // Serial.println(dataString);

    // Gibt den JSON-String zurück
    return dataString;
}

// Gibt werte für die index.html zurück
String Data::getDataForDashboard(const char *settingsPath, const char *lastDataPath)
{
    // Erstellt einen JSON-Buffer
    StaticJsonDocument<200> DashboardData;

    // Erstellt den JSON-Dokument der Einstellungen
    DynamicJsonDocument dataDoc(1024);
    deserializeJson(dataDoc, getData(lastDataPath));

    // Erstellt den JSON-Dokument der Einstellungen
    DynamicJsonDocument settingsDoc(1024);
    deserializeJson(settingsDoc, getData(settingsPath));

    // Überprüft ob SD-Karte vorhanden ist
    bool isFileExist = isSDExists();

    if (!isFileExist)
    {
        // Serial Monitor ausgabe
        // Serial.print("SD exist: ");
        // Serial.println(isFileExist);

        // Erstellt den JSON-String
        String dashboardString = "no data";

        return dashboardString;
    }
    else
    {
        // Serial Monitor ausgabe
        // Serial.print("SD exist: ");
        // Serial.println(isFileExist);

        // Enthält den angle-Wert
        DashboardData["tiltValue"] = String(dataDoc["angle"].as<float>(), 2) + " °";

        // Enthält den temperature-Wert
        if (settingsDoc["temp_unit"] == "C")
        {
            float temp = dataDoc["temperature"].as<float>();
            if (dataDoc["temp_units"] == "F")
                temp = (temp - 32) * 5 / 9;
            else if (dataDoc["temp_units"] == "K")
                temp -= 273.15;
            DashboardData["tempValue"] = String(temp, 2) + " °Celsius";
        }
        else if (settingsDoc["temp_unit"] == "F")
        {
            float temp = dataDoc["temperature"].as<float>();
            if (dataDoc["temp_units"] == "C")
                temp = temp * 9 / 5 + 32;
            else if (dataDoc["temp_units"] == "K")
                temp = (temp - 273.15) * 9 / 5 + 32;
            DashboardData["tempValue"] = String(temp, 2) + " °Fahrenheit";
        }
        else if (settingsDoc["temp_unit"] == "K")
        {
            float temp = dataDoc["temperature"].as<float>();
            if (dataDoc["temp_units"] == "C")
                temp += 273.15;
            else if (dataDoc["temp_units"] == "F")
                temp = (temp - 32) * 5 / 9 + 273.15;
            DashboardData["tempValue"] = String(temp, 2) + " Kelvin";
        }

        // Enthält den battery-Wert
        DashboardData["batteryValue"] = String(dataDoc["battery"].as<float>(), 2) + " V";

        // Enthält den gravity-Wert
        if (settingsDoc["gravity_unit"] == "P")
        {
            DashboardData["gravityValue"] = String(dataDoc["gravity"].as<float>(), 2) + " °P";
        }
        else if (settingsDoc["gravity_unit"] == "SG")
        {
            float SG = 1 + (dataDoc["gravity"].as<float>() / (258.6 - ((dataDoc["gravity"].as<float>() / 258.2) * 227.1)));
            DashboardData["gravityValue"] = String(SG, 3) + " SG";
        }
        // Enthält den SDExists-Wert
        DashboardData["sdValue"] = bool(isFileExist);

        // Erstellt den JSON-String
        String dashboardString;
        serializeJson(DashboardData, dashboardString);

        // Serial Monitor Ausgabe
        // Serial.print("Dashboard data :");
        // Serial.println(dashboardString);

        // Gibt den JSON-String zurück
        return dashboardString;
    }
}

// Rückgabe von Daten der Dateien nach Dateipfad
String Data::getData(const char *path)
{
    // Startet das Filesystem
    startFS();

    // Generiert einen String der Daten der SD-Karte
    String dataString = getDataFromFile(SD, path);

    // Beendet das Filesystem
    endFS();

    // Gibt den JSON-String zurück
    return dataString;
}

// Läd alle Daten von der SD-Karte
String Data::getDataFromFile(fs::FS &fs, const char *path)
{
    Serial.printf("Reading file: %s\r\n", path);

    File file = fs.open(path);
    if (!file || file.isDirectory())
    {
        Serial.println("- failed to open file for reading");
        return String();
    }

    String fileContent;
    while (file.available())
    {
        fileContent += file.readStringUntil('\n');
        break;
    }
    file.close();
    Serial.println(fileContent);
    return fileContent;
}

// Daten an den Speicher senden
void Data::appendData(String data, const char *path)
{
    // Startet das Filesystem
    startFS();

    // Speichert die Daten auf die SD-Karte
    safeDataToFile(SD, path, data.c_str());

    // Beendet das Filesystem
    endFS();
}

// Daten aus dem Speicher löschen
void Data::deleteAllData(const char *path)
{
    // Startet das Filesystem
    startFS();

    // Löscht die Daten auf der SD-Karte
    deleteDataFromFile(SD, path);

    // Beendet das Filesystem
    endFS();
}

// Speichert Daten in der SD-Karte
void Data::safeDataToFile(fs::FS &fs, const char *path, const char *message)
{
    Serial.printf("Appending to file: %s\r\n", path);

    File file = fs.open(path, FILE_APPEND);
    if (!file)
    {
        Serial.println("- failed to open file for appending");
        return;
    }
    if (file.print(message))
    {
        Serial.println("- message appended");
    }
    else
    {
        Serial.println("- append failed");
    }
    file.close();
}

// Löscht Daten in der SD-Karte
void Data::deleteDataFromFile(fs::FS &fs, const char *path)
{
    Serial.printf("Deleting file: %s\r\n", path);

    if (fs.remove(path))
    {
        Serial.println("- file deleted");
    }
    else
    {
        Serial.println("- delete failed");
    }
}

// Läd Daten aus der SD-Karte als Json
void Data::requestDataFromFile(AsyncWebServerRequest *request, const char *path)
{
    startFS();
    request->send(SD, path);
    endFS();
}

// Läd Daten aus der SD-Karte als download
void Data::downloadDataFromFile(AsyncWebServerRequest *request, const char *path)
{
    startFS();
    request->send(SD, path, "text/plain", true);
    endFS();
}

// Start des SD-Filesystem
void Data::startFS()
{
    if (!SD.begin(SD_CS))
    {
        Serial.println("An error has occurred while mounting SD-filesystem");
    }
    else
    {
        Serial.println("SD-filesystem mounted successfully");
        return;
    }
}

// Ende des SD-Filesystem
void Data::endFS()
{
    SD.end();
}

// Gibt reduzierte Daten der iSindel weiter
void Data::initSettings(const char *settingsPath)
{
    if (!isSDExists())
    {
        Serial.println("No SD card atached!");
    }
    else
    {
        if (!isDataFileExists(settingsPath))
        {
            // Erstellt einen JSON-Buffer
            StaticJsonDocument<200> settings;

            // Fügt Einträge zum JSON-Buffer hinzu
            settings["gravity_unit"] = "P";
            settings["temp_unit"] = "C";
            settings["language"] = "de";

            // Erstellt den JSON-String
            String settingsString;
            serializeJson(settings, settingsString);

            // Einstellungen speichern
            appendData(settingsString, settingsPath);

            Serial.println("Set default settings");
        }
        else
        {
            Serial.println("Settings already available");
        }
    }
}

// prüft ob Datei auf SD-Karte vorhanden ist
bool Data::isDataFileExists(const char *path)
{
    startFS();

    if (!SD.exists(path))
    {
        Serial.println("Data file not found");
        return false;
    }

    return true;

    endFS();
}

// prüft ob SD-Karte vorhanden ist
bool Data::isSDExists()
{

    if (!SD.begin(SD_CS))
    {
        Serial.println("No SD card exists!");
        return false;
    }
    Serial.println("SD card exists!");
    endFS();
    return true;
}