/**************************************************************

    "iMonitor"
    By Adrian S. <>

 **************************************************************/

#include <TFT_mask.h>
#include <TFT_eSPI.h>

TFT_eSPI tft = TFT_eSPI();

// Create an instance of the TFT_eSprite class
TFT_eSprite sprite = TFT_eSprite(&tft);

// Define screen dimensions
#define SCREEN_WIDTH 160
#define SCREEN_HEIGHT 128

// Define colors
#define BLACK TFT_BLACK
#define WHITE TFT_WHITE
#define RED TFT_RED
#define BLUE TFT_BLUE
#define DARKGREEN TFT_DARKGREEN
#define GREEN TFT_GREEN
#define GREENYELLOW TFT_GREENYELLOW
#define ORANGE TFT_ORANGE
#define DARKORANGE 0xFFA500
#define YELLOW TFT_YELLOW

// Define font sizes
#define FONT_SIZE_LARGE 2
#define FONT_SIZE_SMALL 1

TFT_mask::TFT_mask()
{
}

void TFT_mask::test()
{
  tft.setRotation(3); // Clear the screen with a white background
  tft.fillScreen(TFT_WHITE);
}

void TFT_mask::initTFT()
{
  tft.init();
  Serial.println("Init TFT-Display done!");
  startDisplay();
  delay(60);
  Serial.println("iMonitor - Device has started");
}

void TFT_mask::startDisplay()
{
  tft.setRotation(3);
  tft.fillScreen(TFT_BLACK);
  tft.drawRoundRect(0, 0, 160, 128, 2, TFT_WHITE);
  tft.setTextSize(3);
  tft.setTextColor(TFT_WHITE);
  tft.setCursor(7, 53);
  tft.print("iMonitor");
}

void TFT_mask::setNoData()
{
  tft.setRotation(3);
  tft.fillScreen(TFT_BLACK);
  tft.drawRoundRect(0, 0, 160, 128, 2, TFT_WHITE);
  tft.setTextSize(3);
  tft.setTextColor(TFT_WHITE);
  tft.setCursor(7, 43);
  tft.print("iMonitor");
  tft.setTextSize(1);
  tft.setCursor(60, 85);
  tft.print("no data");
  Serial.println("iMonitor - no data");
}

void TFT_mask::updateTFT(String data, String settings, bool SD)
{

  // Erstelle den JSON-Dokument der iSpindel-Daten
  DynamicJsonDocument iMonitorDoc(1024);
  deserializeJson(iMonitorDoc, data);

  // Erstelle den JSON-Dokument der Einstellungen
  DynamicJsonDocument settingsDoc(1024);
  deserializeJson(settingsDoc, settings);

  const char *name = iMonitorDoc["name"];
  double temp_float = iMonitorDoc["temperature"];
  double battery_float = iMonitorDoc["battery"];
  double gravity_float = iMonitorDoc["gravity"];
  int RSSI = iMonitorDoc["RSSI"];
  String temp_unit = iMonitorDoc["temp_units"];
  String settings_temp_unit;
  String gravity_unit;

  if (SD)
  {
    settings_temp_unit = settingsDoc["temp_unit"].as<String>();
    gravity_unit = settingsDoc["gravity_unit"].as<String>();
  }
  else
  {
    settings_temp_unit = "C";
    gravity_unit = "P";
  }

  tft.setRotation(3);
  tft.fillScreen(TFT_BLACK);

  updateName(name);
  updateSD(SD);
  updateGravity(gravity_float, gravity_unit);
  updateTemperature(temp_float, temp_unit, settings_temp_unit);
  updateBattery(battery_float);
  updateRSSI(RSSI);
}

void TFT_mask::updateBattery(double value)
{
  double battery_value = (value - 3.50) / (4.2 - 3.5) * 100;
  int battery_int_value = int(battery_value);

  tft.fillRect(3, SCREEN_HEIGHT - 18, SCREEN_WIDTH - 40, 16, BLACK);
  int sub_battery_width = (battery_int_value - 1) / 99.0 * (SCREEN_WIDTH - 40) + 2;

  if (battery_int_value >= 80)
  {
    // Farbe für 80-100%
    tft.drawRect(3, SCREEN_HEIGHT - 18, SCREEN_WIDTH - 40, 16, DARKGREEN);
    tft.drawRect(4, SCREEN_HEIGHT - 17, SCREEN_WIDTH - 42, 14, DARKGREEN);
    tft.fillRect(3, SCREEN_HEIGHT - 18, sub_battery_width, 16, DARKGREEN);
  }
  else if (battery_int_value >= 60)
  {
    // Farbe für 60-79%
    tft.drawRect(3, SCREEN_HEIGHT - 18, SCREEN_WIDTH - 40, 16, GREENYELLOW);
    tft.drawRect(4, SCREEN_HEIGHT - 17, SCREEN_WIDTH - 42, 14, GREENYELLOW);
    tft.fillRect(3, SCREEN_HEIGHT - 18, sub_battery_width, 16, GREENYELLOW);
  }
  else if (battery_int_value >= 40)
  {
    // Farbe für 40-59%
    tft.drawRect(3, SCREEN_HEIGHT - 18, SCREEN_WIDTH - 40, 16, ORANGE);
    tft.drawRect(4, SCREEN_HEIGHT - 17, SCREEN_WIDTH - 42, 14, ORANGE);
    tft.fillRect(3, SCREEN_HEIGHT - 18, sub_battery_width, 16, ORANGE);
  }
  else
  {
    // Farbe für 0-39%
    tft.drawRect(3, SCREEN_HEIGHT - 18, SCREEN_WIDTH - 40, 16, RED);
    tft.drawRect(4, SCREEN_HEIGHT - 17, SCREEN_WIDTH - 42, 14, RED);
    tft.fillRect(3, SCREEN_HEIGHT - 18, sub_battery_width, 16, RED);
  }

  tft.setTextSize(1);
  tft.setTextColor(WHITE);
  tft.setCursor(SCREEN_WIDTH - 32, SCREEN_HEIGHT - 13);
  tft.print(battery_value, 0);
  tft.print("%");
}

void TFT_mask::updateName(const char *value)
{
  tft.fillRect(0, 0, SCREEN_WIDTH - 21, 20, BLACK);
  tft.setTextColor(TFT_GOLD);
  tft.setTextSize(1);
  tft.setCursor(3, 6);
  tft.print(value);
  tft.drawFastHLine(3, 20, SCREEN_WIDTH - 6, WHITE);
}

void TFT_mask::updateSD(bool value)
{
  tft.fillRect(SCREEN_WIDTH - 21, 0, 20, 20, BLACK);
  if (value)
  {
    tft.fillRect(SCREEN_WIDTH - 19, 2, 16, 16, DARKGREEN);
  }
  else
  {
    tft.fillRect(SCREEN_WIDTH - 19, 2, 16, 16, RED);
  }
  tft.fillTriangle(SCREEN_WIDTH - 19, 2, SCREEN_WIDTH - 15, 2, SCREEN_WIDTH - 19, 6, BLACK);
  tft.setTextColor(BLACK);
  tft.setTextSize(1);
  tft.setCursor(SCREEN_WIDTH - 16, 8);
  tft.print("SD");
}

void TFT_mask::updateGravity(double value, String unit)
{
  tft.fillRect(0, 22, SCREEN_WIDTH - 8, 44, BLACK);
  tft.setTextColor(TFT_WHITE);
  tft.setTextSize(1);

  if (unit == "P")
  {
    tft.setCursor(3, 24);
    tft.print("Plato");
    tft.setTextSize(3);
    tft.setCursor(40, 36);
    tft.print(value);
  }
  else
  {
    float SG = 1 + (value / (258.6 - ((value / 258.2) * 227.1)));
    tft.setCursor(3, 24);
    tft.print("Specific Gravity");
    tft.setTextSize(3);
    tft.setCursor(40, 36);
    tft.print(SG, 3);
  }
}

void TFT_mask::updateTemperature(double value, String unit, String settingsUnit)
{
  tft.fillRect(0, 66, SCREEN_WIDTH - 8, 44, BLACK);
  tft.setTextColor(TFT_WHITE);
  tft.setTextSize(1);

  double convertedValue = 0;

  if (unit == settingsUnit)
  {
    convertedValue = value;
  }
  else if (unit == "C")
  {
    if (settingsUnit == "F")
    {
      convertedValue = value * 1.8 + 32;
    }
    else if (settingsUnit == "K")
    {
      convertedValue = value + 273.15;
    }
  }
  else if (unit == "F")
  {
    if (settingsUnit == "C")
    {
      convertedValue = (value - 32) / 1.8;
    }
    else if (settingsUnit == "K")
    {
      convertedValue = (value - 32) / 1.8 + 273.15;
    }
  }
  else if (unit == "K")
  {
    if (settingsUnit == "C")
    {
      convertedValue = value - 273.15;
    }
    else if (settingsUnit == "F")
    {
      convertedValue = (value - 273.15) * 1.8 + 32;
    }
  }

  String tempUnit;

  if (settingsUnit == "C")
  {
    tempUnit = "Celsius";
  }
  else if (settingsUnit == "F")
  {
    tempUnit = "Fahrenheit";
  }
  else if (settingsUnit == "K")
  {
    tempUnit = "Kelvin";
  }

  tft.setCursor(3, 68);
  tft.print(tempUnit);
  tft.setTextSize(3);
  tft.setCursor(40, 80);
  tft.print(convertedValue, 1);
}

void TFT_mask::updateRSSI(int value)
{
  tft.fillRect(SCREEN_WIDTH - 7, 23, 4, SCREEN_HEIGHT - 25, BLACK);

  if (value <= -70)
  {
    tft.fillRect(SCREEN_WIDTH - 7, (SCREEN_HEIGHT - 23) * 3 / 4 + 23, 4, (SCREEN_HEIGHT - 25) * 1 / 4, RED);
  }
  else if (value <= -60)
  {
    tft.fillRect(SCREEN_WIDTH - 7, (SCREEN_HEIGHT - 23) / 2 + 23, 4, (SCREEN_HEIGHT - 25) * 2 / 4, ORANGE);
  }
  else if (value <= -50)
  {
    tft.fillRect(SCREEN_WIDTH - 7, (SCREEN_HEIGHT - 23) / 4 + 23, 4, (SCREEN_HEIGHT - 25) * 3 / 4, GREENYELLOW);
  }
  else
  {
    tft.fillRect(SCREEN_WIDTH - 7, 23, 4, SCREEN_HEIGHT - 25, DARKGREEN);
  }
}