/**************************************************************

    "iMonitor"
    By Adrian S. <>

 **************************************************************/

#include <Arduino.h>
#include <TFT_mask.h> 
#include <SPI.h>
#include <WiFi.h>
#include <AsyncTCP.h>
#include <ESPAsyncWebServer.h>
#include "SPIFFS.h" 
#include "AsyncJson.h"
#include <Arduino_JSON.h>
#include "SD.h"
#include <Preferences.h>

#define SD_CS 5

JSONVar data_1; 
JSONVar data_2; 
JSONVar data_3;
JSONVar data_4; 
JSONVar readings; 
String incoming_data_1; 
File file_1;
AsyncWebServer server(80);
AsyncEventSource events("/events");
TFT_mask mask = TFT_mask();
Preferences preferences;

const char* dataPath = "/data.txt"; 

const char* name = "iSpindel";
double temp_float;
double battery_float;
double gravity_value;
int RSSI;
int interval;
double angle;
int temp_units = 1;

int gravity_unit = 1;
int language = 1;
int logging = 1;

const char* ssid = "iMonitor";
const char* psk = "iMonitor";

const char* input_parameter_1 = "gravity-unit";
const char* input_parameter_2 = "temp-unit";
const char* input_parameter_3 = "language";
const char* input_parameter_4 = "logging";

void startFS() {

    if(logging == 1){
      if (!SD.begin(SD_CS)) {
        Serial.println("An error has occurred while mounting SD-filesystem");
      } else {
        Serial.println("SD-filesystem mounted successfully");
        return;
      }
    }
}

void endFS() {

    SD.end();
}

// Return a String to visible actual iSpindel data on index.html
String getCurrentValues(){ 

  data_3["tiltValue"] = String(angle) + " °"; 
  if (temp_units == 1){
    data_3["tempValue"] = String(temp_float) + " °C" ; 
  } else if(temp_units == 2){
    data_3["tempValue"] = String(temp_float) + " °F" ; 
  } else {
    data_3["tempValue"] = String(temp_float) + " °K" ; 
  } 

  data_3["batteryValue"] = String(battery_float) + " V"; 

  int gravity = preferences.getInt("gravity_unit");

  if (gravity == 1){
    data_3["gravityValue"] = String(gravity_value) + " °P"; 
  } else if(gravity == 2){
    float SG = 1 + (gravity_value / (258.6 - ( (gravity_value/258.2) *227.1) ) );
    data_3["gravityValue"] = String(SG,3) + " SG"; 
  }

  String jsonString = JSON.stringify(data_3);
  return jsonString; 
} 

String readFile(fs::FS &fs, const char * path){ 
  Serial.printf("Reading file: %s\r\n", path); 
 
  File file = fs.open(path); 
  if(!file || file.isDirectory()){ 
    Serial.println("- failed to open file for reading"); 
    return String(); 
  } 
 
  String fileContent; 
  while(file.available()){ 
    fileContent += file.readStringUntil('\n'); 
    break;
      } 
  file.close(); 
  Serial.println(fileContent);
  return fileContent; 
} 
 
void appendFile(fs::FS &fs, const char * path, const char * message){ 
  Serial.printf("Appending to file: %s\r\n", path); 

  File file = fs.open(path, FILE_APPEND); 
  if(!file){ 
    Serial.println("- failed to open file for appending"); 
    return; 
  } 
  if(file.print(message)){ 
    Serial.println("- message appended"); 
  } else { 
    Serial.println("- append failed"); 
  } 
  file.close(); 
} 
 
void deleteFile(fs::FS &fs, const char * path){ 
  Serial.printf("Deleting file: %s\r\n", path); 
  if(fs.remove(path)){ 
    Serial.println("- file deleted"); 
  } else { 
    Serial.println("- delete failed"); 
  } 
} 
  
int getFileSize(fs::FS &fs, const char * path){ 
  File file = fs.open(path); 
 
  if(!file){ 
    Serial.println("Failed to open file for checking size"); 
    return 0; 
  } 
  Serial.print("File size: "); 
  Serial.println(file.size()); 
 
  return file.size(); 
}
 
// return iSpindel data
String getSensorReadings(){ 
  data_1["angle"] = angle;
  data_1["gravity"] = gravity_value; 
  data_1["temperature"] = temp_float; 
  data_1["battery"] = battery_float; 
  String jsonString = JSON.stringify(data_1); 
  return jsonString; 
}

// return config data
String getConfigData(){ 
  data_2["gravity_unit"] = preferences.getInt("gravity_unit");
  data_2["temp_unit"] = temp_units; 
  data_2["language"] = language;
  data_2["logging"] = logging;
  String jsonString = JSON.stringify(data_2); 
  return jsonString; 
}

// process iSpindel incoming data
void processData() {
  
  DynamicJsonDocument doc(1024);
  deserializeJson(doc, incoming_data_1);

  // Set the temperature unit (incoming unit from the iSpindel device)
  if(doc["temp_units"] == "C"){
    temp_units = 1;
  } else if (doc["temp_units"] == "F") {
    temp_units = 2;
  } else if (doc["temp_units"] == "K") {
    temp_units = 3;
  }
  
  float gravity = doc["gravity"];
  int gravity_int = gravity*100;
  gravity_value = double(gravity_int)/100;
  
  float temp = doc["temperature"];
  int temp_int = temp*100;
  temp_float = double(temp_int)/100;
  
  float battery = doc["battery"];
  int battery_int = battery*100;
  battery_float = double(battery_int)/100;
  
  name = doc["name"];
  RSSI = doc["RSSI"];
  angle = doc["angle"];
  interval = doc["interval"];

  // Make JSON-string for the TFT-mask
  data_4["gravity"] = gravity_value; 
  data_4["temperature"] = temp_float; 
  data_4["battery"] = battery_float; 
  data_4["name"] = name; 
  data_4["RSSI"] = RSSI; 
  data_4["temp_units"] = temp_units;
  data_4["gravity_units"] = preferences.getInt("gravity_unit");

  String jsonString = JSON.stringify(data_4); 


  mask.updateTFT(jsonString);
}

// Replaces placeholder with value 
String processor_conf(const String& var){ 


  if(var == "log_check_SD") { 
    if(logging == 1) { 
      return "SD-Card selected";
    } 
    else { 
      return "no logging selected";
    }  
  } 
  return String(); 

  if(var == "language_de") { 
    if(language == 1) { 
      return "German selected";
    } 
    else { 
      return "English selected";
    }  
  } 
  return String(); 
} 

void setup() {
  Serial.begin(115200);

  preferences.begin("config", false);

  preferences.getInt("gravity_unit", gravity_unit);
 
  WiFi.mode(WIFI_AP);
  WiFi.softAP(ssid, psk);

  // Initialize TFT-Display
  mask.initTFT();

  // Initialize SPIFFS
  if(!SPIFFS.begin(true)){
    Serial.println("An Error has occurred while mounting SPIFFS");
    return;
  }

  preferences.begin("config_data", false);
  
  server.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send(SPIFFS, "/index.html", "text/html");
  });
  
  server.on("/display", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send(SPIFFS, "/display.html", "text/html");
  });
  
  server.on("/configuration", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send(SPIFFS, "/configuration.html", "text/html", false, processor_conf);
  });
  
  server.on("/maintenance", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send(SPIFFS, "/maintenance.html", "text/html");
  });
  
  server.on("/readings", HTTP_GET, [](AsyncWebServerRequest *request){ 
    startFS();

    if(logging == 1){
      request->send(SD, "/data.txt", "text/txt");
      endFS();
    } 
  }); 

  server.on("/view-data", HTTP_GET, [](AsyncWebServerRequest *request){ 
    startFS();

    if(logging == 1){ 
      request->send(SD, dataPath);

      endFS();
      Serial.println("Data has been send!");
    }
  }); 
  
  // Get values to display actual iSpindel data on index.html
  server.on("/values", HTTP_GET, [](AsyncWebServerRequest *request){
    String json = getCurrentValues(); 
    request->send(200, "application/json", json); 
    json = String(); 
  });

  // process incoming values from the configuration.html form
  server.on("/safeconfig", HTTP_GET, [](AsyncWebServerRequest *request){ 

  Serial.println("New configuration data safed!");

  String gravity_unit_input;
  String language_input;
  String logging_input;
  
  gravity_unit_input = request->getParam(input_parameter_1)->value();
  language_input = request->getParam(input_parameter_3)->value();
  logging_input = request->getParam(input_parameter_4)->value();

    if (gravity_unit_input == "1") {
      preferences.putInt("gravity_unit", 1);
      Serial.print("<gravity unit is: Plato>");
    } else {
      preferences.putInt("gravity_unit", 2);
      Serial.print("<gravity unit is: Specific Gravity>");
    }
    Serial.print(", ");
    if (language_input == "1") {
      preferences.putInt("language", 1);
      Serial.print("<language is: German>");
    } else {
      preferences.putInt("language", 2);
      Serial.print("<language is: English>");
    }
    Serial.print(", ");
    if (logging_input == "1") {
      Serial.println("<logging on! .: SD-Card :.>");
    } 
  

  request->send(200, "text/html", "OK");
  }); 

  server.on("/delete-data", HTTP_GET, [](AsyncWebServerRequest *request){ 
    startFS();

    if(logging == 1){
      deleteFile(SD, dataPath);
      endFS();
    }
    
    request->send(200, "text/plain", "data.txt has been deleted."); 
    delay(1000);
    request->redirect("/");
  }); 

  // Request from iSpindel_data device
  server.on("/iMonitor", HTTP_POST, 
      [](AsyncWebServerRequest *request)
      {
        
      },NULL,
      [](AsyncWebServerRequest *request, uint8_t *data, size_t len, size_t index, size_t total)
      {
        incoming_data_1 = (char*)data;
        processData();
        events.send(getSensorReadings().c_str(),"new_data_1" ,millis());
        
        String message = getSensorReadings() + ",";

        if(logging == 1){
          startFS();
          appendFile(SD, "/data.txt", message.c_str());
          uint64_t cardSize = SD.cardSize();
          int cardSizeInMB = cardSize/(1024 * 1024);
          Serial.println("Append SD-file");
          Serial.print("Card size: ");
          Serial.print(cardSizeInMB);
          Serial.println(" MB");
          endFS();
        } else {
          Serial.println("Error to append to file!");
        }

      
          });
      
  server.serveStatic("/", SPIFFS, "/");

  server.begin();

}

void loop() {
}
